<?php
/**
 * @author Daniel J. Summers <daniel@bitbadger.solutions>
 * @license MIT
 */

declare(strict_types=1);

namespace BitBadger\PDODocument;

/**
 * Functions to patch (partially update) documents
 */
class Patch
{
    /**
     * Patch a document by its ID
     *
     * @param string $tableName The table in which the document should be patched
     * @param mixed $docId The ID of the document to be patched
     * @param mixed[]|object $patch The object with which the document should be patched (will be JSON-encoded)
     * @throws DocumentException If any is encountered (database mode must be set)
     */
    public static function byId(string $tableName, mixed $docId, array|object $patch): void
    {
        Custom::nonQuery(Query\Patch::byId($tableName, $docId),
            array_merge(Parameters::id($docId), Parameters::json(':data', $patch)));
    }

    /**
     * Patch documents using a comparison on JSON fields
     *
     * @param string $tableName The table in which documents should be patched
     * @param Field[] $fields The field comparison to match
     * @param mixed[]|object $patch The object with which the documents should be patched (will be JSON-encoded)
     * @param FieldMatch|null $match How to handle multiple conditions (optional; defaults to All)
     * @throws DocumentException If any is encountered
     */
    public static function byFields(string $tableName, array $fields, array|object $patch,
                                    ?FieldMatch $match = null): void
    {
        Parameters::nameFields($fields);
        Custom::nonQuery(Query\Patch::byFields($tableName, $fields, $match),
            Parameters::addFields($fields, Parameters::json(':data', $patch)));
    }

    /**
     * Patch documents using a JSON containment query (`@>`; PostgreSQL only)
     *
     * @param string $tableName The table in which documents should be patched
     * @param mixed[]|object $criteria The JSON containment query values to match
     * @param mixed[]|object $patch The object with which the documents should be patched (will be JSON-encoded)
     * @throws DocumentException If the database mode is not PostgreSQL, or if an error occurs
     */
    public static function byContains(string $tableName, array|object $criteria, array|object $patch): void
    {
        Custom::nonQuery(Query\Patch::byContains($tableName),
            array_merge(Parameters::json(':criteria', $criteria), Parameters::json(':data', $patch)));
    }

    /**
     * Patch documents using a JSON Path match query (`@?`; PostgreSQL only)
     *
     * @param string $tableName The table in which documents should be patched
     * @param string $path The JSON Path match string
     * @param mixed[]|object $patch The object with which the documents should be patched (will be JSON-encoded)
     * @throws DocumentException If the database mode is not PostgreSQL, or if an error occurs
     */
    public static function byJsonPath(string $tableName, string $path, array|object $patch): void
    {
        Custom::nonQuery(Query\Patch::byJsonPath($tableName),
            array_merge([':path' => $path], Parameters::json(':data', $patch)));
    }
}
